==> jaguar_mux-0.0.1.tar.gz: lib/src/mux/mux.dart <==
library jaguar_mux.mux;

import 'dart:async';
import 'dart:io';
import 'package:jaguar/jaguar.dart' as j;
import 'package:jaguar_mux/jaguar_mux.dart';

import 'dart:mirrors';
import 'package:quiver/core.dart';

part 'inputs/inject.dart';
part 'inputs/input.dart';
part 'interceptor/interceptor.dart';

class _Inter {
  final _Interceptor wrapper;

  final j.Interceptor inter;

  final InstanceMirror interMirror;

  _Inter(this.wrapper, this.inter, this.interMirror);
}

class _Route implements j.RequestHandler {
  final j.RouteBase route;

  final Function handler;

  final ClosureMirror _invoker;

  final List<Inject> _required;

  final Set<String> _optional;

  final List<_Interceptor> wrappers;

  _Route(this.route, this.handler, this._invoker, this._required,
      this._optional, this.wrappers);

  Future<bool> handleRequest(HttpRequest request, {String prefix}) async {
    j.PathParams pathParams = new j.PathParams();
    if (!route.match(request.uri.path, request.method, prefix, pathParams)) {
      return false;
    }

    j.QueryParams queryParams = new j.QueryParams(request.uri.queryParameters);

    j.Response response = new j.Response(null,
        statusCode: route.statusCode, headers: route.headers);

    final List<_Inter> inters = [];
    final Map<InputInject, dynamic> results = {};
    try {
      for (_Interceptor wrapper in wrappers) {
        final j.Interceptor i = wrapper.routeWrapper.createInterceptor();
        final InstanceMirror im = reflect(i);
        final _Inter inter = new _Inter(wrapper, i, im);
        inters.add(inter);

        if (inter.wrapper._pre == null) continue;

        final List reqParam = [];

        for (Inject inj in inter.wrapper._pre.injects) {
          reqParam.add(_makeParam(inj, request, response, results, pathParams));
        }

        dynamic result = inter.interMirror.invoke(#pre, reqParam).reflectee;
        if (result is Future) {
          result = await result;
        }
        InputInject key = new InputInject(wrapper.interceptorType,
            id: wrapper.routeWrapper.id);
        results[key] = result;
      }

      {
        final List reqParam = [];
        for (Inject inj in _required) {
          reqParam.add(_makeParam(inj, request, response, results, pathParams));
        }

        final Map<Symbol, dynamic> optParams = {};
        for (String p in _optional) {
          optParams[new Symbol(p)] = queryParams.getField(p);
        }

        final InstanceMirror resultIm = _invoker.apply(reqParam, optParams);
        if (resultIm.hasReflectee) {
          response.value = resultIm.reflectee;
        }
      }

      // Interceptors post
      for (_Inter inter in inters.reversed) {
        if (inter.wrapper._post == null) continue;

        final List reqParam = [];

        for (Inject inj in inter.wrapper._post.injects) {
          reqParam.add(_makeParam(inj, request, response, results, pathParams));
        }

        dynamic resp = inter.interMirror.invoke(#post, reqParam).reflectee;
        if (resp is Future) {
          resp = await resp;
        }
        if (resp is j.Response) {
          response = resp;
        }
      }

      await response.writeResponse(request.response);
    } catch (e) {
      for (_Inter inter in inters.reversed) {
        await inter.inter.onException();
      }
      rethrow;
    }

    return true;
  }

  dynamic _makeParam(Inject inj, HttpRequest request, j.Response response,
      Map<InputInject, dynamic> interceptorResults, j.PathParams pathParams) {
    if (inj is InputInject) {
      if (!interceptorResults.containsKey(inj)) {
        throw new Exception('Interceptor not found for Input!');
      }
      dynamic result = interceptorResults[inj];
      return result;
    } else if (inj is RouteResponseInject) {
      return response;
    } else if (inj is HeaderInject) {
      return request.headers.value(inj.key);
    } else if (inj is HeadersInject) {
      return request.headers;
    } else if (inj is CookieInject) {
      return request.cookies
          .firstWhere((cookie) => cookie.name == inj.key, orElse: () => null)
          ?.value;
    } else if (inj is CookiesInject) {
      return request.cookies;
    } else if (inj is PathVarInject) {
      return pathParams.getField(inj.key);
    } else {
      throw new Exception('Unknown inject to post interceptor method!');
    }
  }

  factory _Route.build(Route route) {
    final InstanceMirror im = reflect(route.handler);

    if (im is! ClosureMirror) {
      throw new Exception('Route handler must be a closure or function!');
    }

    final ClosureMirror c = im as ClosureMirror;
    final MethodMirror m = c.function;

    j.Route jRoute = new j.Route(
        path: route.path, methods: route.methods, pathRegEx: route.pathRegEx);

    final List<Inject> required = _parseReqParamsForRoute(m);

    final Set<String> optional = _parseOptParamsForRoute(m);

    final List<_Interceptor> interceptors = route.interceptors
        .map((j.RouteWrapper inter) => new _Interceptor.build(inter))
        .toList();

    return new _Route(
        jRoute, route.handler, c, required, optional, interceptors);
  }
}

class Mux implements j.RequestHandler {
  final List<_Route> _routes;

  Mux._(this._routes) {}

  Future<bool> handleRequest(HttpRequest request, {String prefix}) async {
    for (_Route route in _routes) {
      if (await route.handleRequest(request, prefix: prefix)) {
        return true;
      }
    }

    return false;
  }

  factory Mux.build(MuxBuilder builder) {
    final List<_Route> routes = [];
    for (Route route in builder.routes) {
      routes.add(new _Route.build(route));
    }
    return new Mux._(routes);
  }
}

==> jaguar_mux-0.0.2.tar.gz: lib/src/mux/mux.dart <==
library jaguar_mux.mux;

import 'dart:async';
import 'dart:io';
import 'package:jaguar/jaguar.dart' as j;
import 'package:jaguar_mux/jaguar_mux.dart';

import 'dart:mirrors';
import 'package:quiver/core.dart';

part 'inputs/inject.dart';
part 'inputs/input.dart';
part 'interceptor/interceptor.dart';

class _Inter {
  final _Interceptor wrapper;

  final j.Interceptor inter;

  final InstanceMirror interMirror;

  _Inter(this.wrapper, this.inter, this.interMirror);
}

class _Route implements j.RequestHandler {
  final j.RouteBase route;

  final Function handler;

  final ClosureMirror _invoker;

  final List<Inject> _required;

  final Set<String> _optional;

  final List<_Interceptor> wrappers;

  _Route(this.route, this.handler, this._invoker, this._required,
      this._optional, this.wrappers);

  Future<bool> handleRequest(HttpRequest request, {String prefix}) async {
    j.PathParams pathParams = new j.PathParams();
    if (!route.match(request.uri.path, request.method, prefix, pathParams)) {
      return false;
    }

    j.QueryParams queryParams = new j.QueryParams(request.uri.queryParameters);

    j.Response response = new j.Response(null,
        statusCode: route.statusCode, headers: route.headers);

    final List<_Inter> inters = [];
    final Map<InputInject, dynamic> results = {};
    try {
      for (_Interceptor wrapper in wrappers) {
        final j.Interceptor i = wrapper.routeWrapper.createInterceptor();
        final InstanceMirror im = reflect(i);
        final _Inter inter = new _Inter(wrapper, i, im);
        inters.add(inter);

        if (inter.wrapper._pre == null) continue;

        final List reqParam = [];

        for (Inject inj in inter.wrapper._pre.injects) {
          reqParam.add(_makeParam(inj, request, response, results, pathParams));
        }

        dynamic result = inter.interMirror.invoke(#pre, reqParam).reflectee;
        if (result is Future) {
          result = await result;
        }
        InputInject key = new InputInject(wrapper.interceptorType,
            id: wrapper.routeWrapper.id);
        results[key] = result;
      }

      {
        final List reqParam = [];
        for (Inject inj in _required) {
          reqParam.add(_makeParam(inj, request, response, results, pathParams));
        }

        final Map<Symbol, dynamic> optParams = {};
        for (String p in _optional) {
          optParams[new Symbol(p)] = queryParams.getField(p);
        }

        dynamic result = _invoker.apply(reqParam, optParams).reflectee;
        if (result is Future) {
          result = await result;
        }
        response.value = result;
      }

      // Interceptors post
      for (_Inter inter in inters.reversed) {
        if (inter.wrapper._post == null) continue;

        final List reqParam = [];

        for (Inject inj in inter.wrapper._post.injects) {
          reqParam.add(_makeParam(inj, request, response, results, pathParams));
        }

        dynamic resp = inter.interMirror.invoke(#post, reqParam).reflectee;
        if (resp is Future) {
          resp = await resp;
        }
        if (resp is j.Response) {
          response = resp;
        }
      }

      await response.writeResponse(request.response);
    } catch (e) {
      for (_Inter inter in inters.reversed) {
        await inter.inter.onException();
      }
      rethrow;
    }

    return true;
  }

  dynamic _makeParam(Inject inj, HttpRequest request, j.Response response,
      Map<InputInject, dynamic> interceptorResults, j.PathParams pathParams) {
    if (inj is InputInject) {
      if (!interceptorResults.containsKey(inj)) {
        throw new Exception('Interceptor not found for Input!');
      }
      dynamic result = interceptorResults[inj];
      return result;
    } else if (inj is RouteResponseInject) {
      return response;
    } else if (inj is HeaderInject) {
      return request.headers.value(inj.key);
    } else if (inj is HeadersInject) {
      return request.headers;
    } else if (inj is CookieInject) {
      return request.cookies
          .firstWhere((cookie) => cookie.name == inj.key, orElse: () => null)
          ?.value;
    } else if (inj is CookiesInject) {
      return request.cookies;
    } else if (inj is PathVarInject) {
      return pathParams.getField(inj.key);
    } else {
      throw new Exception('Unknown inject to post interceptor method!');
    }
  }

  factory _Route.build(Route route) {
    final InstanceMirror im = reflect(route.handler);

    if (im is! ClosureMirror) {
      throw new Exception('Route handler must be a closure or function!');
    }

    final ClosureMirror c = im as ClosureMirror;
    final MethodMirror m = c.function;

    j.Route jRoute = new j.Route(
        path: route.path, methods: route.methods, pathRegEx: route.pathRegEx);

    final List<Inject> required = _parseReqParamsForRoute(m);

    final Set<String> optional = _parseOptParamsForRoute(m);

    final List<_Interceptor> interceptors = route.interceptors
        .map((j.RouteWrapper inter) => new _Interceptor.build(inter))
        .toList();

    return new _Route(
        jRoute, route.handler, c, required, optional, interceptors);
  }
}

class Mux implements j.RequestHandler {
  final List<_Route> _routes;

  Mux._(this._routes) {}

  Future<bool> handleRequest(HttpRequest request, {String prefix}) async {
    for (_Route route in _routes) {
      if (await route.handleRequest(request, prefix: prefix)) {
        return true;
      }
    }

    return false;
  }

  factory Mux.build(MuxBuilder builder) {
    final List<_Route> routes = [];
    for (Route route in builder.routes) {
      routes.add(new _Route.build(route));
    }
    return new Mux._(routes);
  }
}
